// Shell link utility
// by sidenet.ddo.jp
// Licenced under GPL

//Includes
#include <windows.h>
#include <shlobj.h>
#include <iostream>
#include <cstring>
#include <imagehlp.h>

//CreateShortCut Functions

int CreateShortCut(char *FileName, char *Path, char *Args, char *WorkPath, char *Desc)
{

	IShellLink *ShellLink;
	IPersistFile *PersistFile;
	wchar_t wFileName[MAX_PATH];
	
	MultiByteToWideChar(CP_ACP,0,FileName,-1,wFileName,MAX_PATH);
	
	CoInitialize(NULL); //Initialise OLE

	if(!SUCCEEDED(CoCreateInstance(CLSID_ShellLink,NULL,CLSCTX_INPROC_SERVER,IID_IShellLink,(void **)&ShellLink))){
		return 1;
	}else{

		if(!SUCCEEDED(ShellLink->QueryInterface(IID_IPersistFile,(void **)&PersistFile))){
			return 1;
		}else{

		ShellLink->SetPath(Path);
		ShellLink->SetArguments(Args);
		ShellLink->SetWorkingDirectory(WorkPath);
		ShellLink->SetDescription(Desc);
            
  		PersistFile->Save(wFileName,TRUE);
		PersistFile->Release();
		
		ShellLink->Release();
	
		CoUninitialize();
		}
	}
	return 0;
}


int SearchString(char *Target,char *String){
	int Counter;
	int Target_Length;
	int String_Length;
	
	Target_Length=strlen(Target);
	String_Length=strlen(String);

	for(Counter=0; Counter<Target_Length; Counter++){
		if(!strncmp(Target,String+Counter,Target_Length)){
			return Counter;
		}
	}

	return -1;
}

 
int Replace(char *From,char *To,char *String){
	int Index; 
	int From_Length;
	int To_Length;
	char Buf[MAX_PATH] = "";
	
	From_Length=strlen(From);
	To_Length=strlen(To);

	Index=SearchString(From,String);
	if(Index==-1){
		return 1;
	}
	
	strcpy(Buf,String+Index+From_Length);
	strcpy(String+Index,To);
	strcpy(String+Index+To_Length,Buf);
	return 0;
}


int main(int argc, char *argv[]){

char DesktopPath[MAX_PATH];
char StartMenuPath[MAX_PATH];
char ProgramsPath[MAX_PATH];
char ShortCutPath[MAX_PATH];
char Arguments[MAX_PATH];
char WorkPath[MAX_PATH];
LPITEMIDLIST pidl;
HWND hWnd = NULL;

if (argc<2){
	printf("Usage:\nshelllink [target.exe] [dest.lnk] [args] [work path]\n\n");
	printf("__desktop__ means desktop path\n__startmenu__ means start menu path.\n\n");
	printf("Example:shellink c:\\windows\\explorer.exe __desktop__\\explorer.lnk\n\n");
	printf("Example2:shellink mkdir __startmenu__\\Programs\\My Apps\\\ncreates directory.\n");
	return 0;
}

strcpy(ShortCutPath,argv[2]);

if (argc>3){
	strcpy(Arguments,argv[3]);
}else{
	strcpy(Arguments,"");
}

if (argc>4){
	strcpy(WorkPath,argv[4]);
}else{
	strcpy(WorkPath,"");
}

SHGetSpecialFolderLocation( hWnd, CSIDL_DESKTOPDIRECTORY, &pidl );
SHGetPathFromIDList( pidl, DesktopPath );

SHGetSpecialFolderLocation( hWnd, CSIDL_STARTMENU, &pidl );
SHGetPathFromIDList( pidl, StartMenuPath );

SHGetSpecialFolderLocation( hWnd, CSIDL_PROGRAMS, &pidl );
SHGetPathFromIDList( pidl, ProgramsPath );


Replace("__desktop__",DesktopPath,ShortCutPath);
Replace("__startmenu__",StartMenuPath,ShortCutPath);
Replace("__programs__",StartMenuPath,ShortCutPath);

//MessageBox(NULL,ShortCutPath,NULL,MB_OK);


if (!strcmp(argv[1],"mkdir")){
	MakeSureDirectoryPathExists(ShortCutPath);
}else{

CreateShortCut(
               ShortCutPath, //target filename
               argv[1],      //dest filename
               Arguments,    //args
               WorkPath,      //work path
               ""            //desc
              ); 
}

return 0;

}

